@@ -3,11 +3,12 @@ package ExtUtils::CBuilder::Base;
 use strict;
 use File::Spec;
 use File::Basename;
+use Cwd ();
 use Config;
 use Text::ParseWords;
 
 use vars qw($VERSION);
-$VERSION = '0.12';
+$VERSION = '0.22';
 
 sub new {
   my $class = shift;
@@ -74,6 +75,11 @@ sub arg_exec_file {
   return ('-o', $file);
 }
 
+sub arg_defines {
+  my ($self, %args) = @_;
+  return map "-D$_=$args{$_}", keys %args;
+}
+
 sub compile {
   my ($self, %args) = @_;
   die "Missing 'source' argument to compile()" unless defined $args{source};
@@ -86,11 +92,13 @@ sub compile {
     (@{$args{include_dirs} || []},
      $self->perl_inc());
   
+  my @defines = $self->arg_defines( %{$args{defines} || {}} );
+  
   my @extra_compiler_flags = $self->split_like_shell($args{extra_compiler_flags});
   my @cccdlflags = $self->split_like_shell($cf->{cccdlflags});
   my @ccflags = $self->split_like_shell($cf->{ccflags});
   my @optimize = $self->split_like_shell($cf->{optimize});
-  my @flags = (@include_dirs, @cccdlflags, @extra_compiler_flags,
+  my @flags = (@include_dirs, @defines, @cccdlflags, @extra_compiler_flags,
 	       $self->arg_nolink,
 	       @ccflags, @optimize,
 	       $self->arg_object_file($args{object_file}),
@@ -147,6 +155,8 @@ sub exe_file {
 
 sub need_prelink { 0 }
 
+sub extra_link_args_after_prelink { return }
+
 sub prelink {
   my ($self, %args) = @_;
   
@@ -158,9 +168,10 @@ sub prelink {
     DL_FUNCS => $args{dl_funcs}     || {},
     FUNCLIST => $args{dl_func_list} || [],
     IMPORTS  => $args{dl_imports}   || {},
-    NAME     => $args{dl_name},
-    DLBASE   => $args{dl_base},
-    FILE     => $args{dl_file},
+    NAME     => $args{dl_name},		# Name of the Perl module
+    DLBASE   => $args{dl_base},		# Basename of DLL file
+    FILE     => $args{dl_file},		# Dir + Basename of symlist file
+    VERSION  => (defined $args{dl_version} ? $args{dl_version} : '0.0'),
   );
   
   # Mksymlists will create one of these files
@@ -191,7 +202,10 @@ sub _do_link {
     $self->prelink(%args,
 		   dl_name => $args{module_name}) if $args{lddl} && $self->need_prelink;
   
-  my @linker_flags = $self->split_like_shell($args{extra_linker_flags});
+  my @linker_flags = ($self->split_like_shell($args{extra_linker_flags}),
+		      $self->extra_link_args_after_prelink(%args, dl_name => $args{module_name},
+							   prelink_res => \@temp_files));
+
   my @output = $args{lddl} ? $self->arg_share_object_file($out) : $self->arg_exec_file($out);
   my @shrp = $self->split_like_shell($cf->{shrpenv});
   my @ld = $self->split_like_shell($cf->{ld});
@@ -227,11 +241,11 @@ sub perl_src {
 
   return unless $ENV{PERL_CORE};
 
-  my $Updir  = File::Spec->updir;
-  my $dir = $Updir;
+  my $Updir = File::Spec->updir;
+  my $dir   = File::Spec->curdir;
 
   # Try up to 5 levels upwards
-  for (1..5) {
+  for (0..10) {
     if (
 	-f File::Spec->catfile($dir,"config_h.SH")
 	&&
@@ -239,14 +253,14 @@ sub perl_src {
 	&&
 	-f File::Spec->catfile($dir,"lib","Exporter.pm")
        ) {
-      return $dir;
+      return Cwd::realpath( $dir );
     }
 
     $dir = File::Spec->catdir($dir, $Updir);
   }
-  
+
   warn "PERL_CORE is set but I can't find your perl source!\n";
-  return;
+  return ''; # return empty string if $ENV{PERL_CORE} but can't find dir ???
 }
 
 # directory of perl's include files
@@ -258,6 +272,7 @@ sub perl_inc {
 
 sub DESTROY {
   my $self = shift;
+  local($., $@, $!, $^E, $?);
   $self->cleanup();
 }
 
